#ifndef MSP_GL_SL_OUTPUT_H_
#define MSP_GL_SL_OUTPUT_H_

#include "compiler.h"
#include "syntax.h"
#include "visitor.h"

namespace Msp {
namespace GL {
namespace SL {

/** Formats the syntax tree as GLSL. */
class Formatter: private TraversingVisitor
{
private:
	Stage *stage;
	std::string formatted;
	unsigned source_index;
	unsigned source_line;
	unsigned indent;
	bool parameter_list;

public:
	Formatter();

	const std::string &apply(Stage &);
	const std::string &apply(Node &n) { n.visit(*this); return formatted; }

private:
	void append(const std::string &);
	void append(char);
	void set_source(unsigned, unsigned);

	virtual void visit(Block &);
	virtual void visit(Literal &);
	virtual void visit(ParenthesizedExpression &);
	virtual void visit(VariableReference &);
	virtual void visit(InterfaceBlockReference &);
	virtual void visit(MemberAccess &);
	virtual void visit(Swizzle &);
	virtual void visit(UnaryExpression &);
	virtual void visit(BinaryExpression &);
	virtual void visit(Assignment &);
	virtual void visit(FunctionCall &);
	virtual void visit(ExpressionStatement &);
	virtual void visit(Import &);
	virtual void visit(Precision &);
	virtual void visit(Layout &);
	virtual void visit(InterfaceLayout &);
	virtual void visit(StructDeclaration &);
	virtual void visit(VariableDeclaration &);
	virtual void visit(InterfaceBlock &);
	virtual void visit(FunctionDeclaration &);
	virtual void visit(Conditional &);
	virtual void visit(Iteration &);
	virtual void visit(Passthrough &);
	virtual void visit(Return &);
	virtual void visit(Jump &);
};

} // namespace SL
} // namespace GL
} // namespace Msp

#endif
